import re, os.path, glob
import maintscript.platforms_install, maintscript.platforms_postinst, maintscript.platforms_postrm, maintscript.platforms_config, maintscript.platforms_templates, maintscript.platforms_dirs, maintscript.platforms_control, maintscript.platforms_lintianoverrides, maintscript.platforms_rules

def get(soup,pif):
    pkg_dir = os.path.join(glob.glob(os.path.expanduser(pif))[0], '')
    # Get platforms list
    platforms_list = soup.findAll('platform') 
    # Show results
    for platform in platforms_list:
        api_level = platform.find('api-level').string
        version =  platform.version.string
        archive = platform.archives.archive.url.string
        revision = re.search("_r[0-9]*",archive).group()[2:]
        sha1 =  platform.archives.archive.checksum.string
        binary = "google-android-platform-"+api_level+"-installer"
        install = pkg_dir+"debian/"+binary+".install"
        postinst = pkg_dir+"debian/"+binary+".postinst"
        postrm = pkg_dir+"debian/"+binary+".postrm"
        config = pkg_dir+"debian/"+binary+".config"
        templates = pkg_dir+"debian/"+binary+".templates"
        dirs = pkg_dir+"debian/"+binary+".dirs"
        overrides = pkg_dir+"debian/"+binary+".lintian-overrides"
        control = pkg_dir+"debian/control"
        rules = pkg_dir+"debian/rules"
        sha1sum = pkg_dir+"for-postinst/default/"+archive+".sha1"
        current_sha1sum = ""

        print(("\033[1;33m- "+platform.description.string+"\033[0m ("+version+")"))

        # Generate/Update <package>.install
        if os.path.isfile(install):
            f = open(install)
            current_sha1sum = re.search("(android|platform)-[0-9]*((.[0-9]*)*)?_r[0-9]*(-linux)?.zip.sha1",f.readlines()[1]).group()
            if current_sha1sum == archive+".sha1":
                print("\033[0;32mOK\033[0m "+binary+".install")
            else:
                print("\033[0;33mOUTDATED\033[0m "+binary+".install")
                f.seek(0)
                i = f.read()
                o = open(install,"w")
                o.write(i.replace(current_sha1sum, archive+".sha1"))
                print(":... \033[0;34mUPDATED\033[0m from "+current_sha1sum+" to "+archive+".sha1")
                o.close()
        else:
            print("\033[0;31mNOT EXIST\033[0m "+binary+".install")
            maintscript.platforms_install.generate(install,api_level,archive)       

        # Generate/Update <archive>.sha1
	current_sha1sum_file = pkg_dir+"for-postinst/default/"+current_sha1sum
        generate_sha1 = False
        if current_sha1sum != "":
            if os.path.isfile(current_sha1sum_file):
                f = open(current_sha1sum_file)
                current_sha1 = re.search(r'\b[0-9a-f]{5,40}\b',f.readlines()[0]).group()
                if current_sha1sum_file != sha1sum:
                    # Remove outdated sha1 file
                    try:
		        os.remove(current_sha1sum_file)
	            except OSError:
		        pass
                    # Generate new sha1 file
                    if os.path.isfile(sha1sum):
                        print("\033[0;32mOK\033[0m "+archive+".sha1")
                    else:
                        generate_sha1 = True
                elif current_sha1 != sha1:
                    generate_sha1 = True
            else:
                generate_sha1 = True
        else:
            generate_sha1 = True

        #Generate SHA1 if needed
        if generate_sha1 == True:
            i = open(pkg_dir+"for-postinst/default/"+archive+".sha1", "w+")
            i.write(sha1+"  "+archive)
            i.close()
            print ":... \033[0;34mGENERATED\033[0m "+archive+".sha1"

        # Generate/Update <package>.postinst
        if os.path.isfile(postinst):
            f = open(postinst)
            match = re.search("r[0-9]+",f.readlines()[7]).group()[1:]
            if int(match) == int(revision):
                print("\033[0;32mOK\033[0m "+binary+".postinst")
            else:
                print("\033[0;33mOUTDATED\033[0m "+binary+".postinst")
                f.seek(0)
                i = f.read()
                o = open(postinst,"w")
                o.write(i.replace(match, revision))
                print(":... \033[0;34mUPDATED\033[0m from revision "+match+" to "+revision)
                o.close()
        else:
            print("\033[0;31mNOT EXIST\033[0m "+binary+".postinst")
            maintscript.platforms_postinst.generate(postinst,api_level,archive)

        #Generate <package>.postrm
        if os.path.isfile(postrm):
           print("\033[0;32mOK\033[0m "+binary+".postrm")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".postrm")
           maintscript.platforms_postrm.generate(postrm,api_level)

        #Generate <package>.config
        if os.path.isfile(config):
           print("\033[0;32mOK\033[0m "+binary+".config")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".config")
           maintscript.platforms_config.generate(config,api_level)

        #Generate <package>.templates
        if os.path.isfile(templates):
           print("\033[0;32mOK\033[0m "+binary+".templates")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".templates")
           maintscript.platforms_templates.generate(templates,api_level)

        #Generate <package>.dirs
        if os.path.isfile(dirs):
           print("\033[0;32mOK\033[0m "+binary+".dirs")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".dirs")
           maintscript.platforms_dirs.generate(dirs,api_level)

        #Generate <package>.lintian-overrides
        if os.path.isfile(overrides):
           print("\033[0;32mOK\033[0m "+binary+".lintian-overrides")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".lintian-overrides")
           maintscript.platforms_lintianoverrides.generate(overrides,api_level)

        #Add package to d/control
        if "platform-"+api_level+"-" in open(control).read():
           print("\033[0;32mOK\033[0m "+binary+" in d/control")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+" in d/control")
           maintscript.platforms_control.generate(control,api_level,archive)

        #Add package to d/rules
        maintscript.platforms_rules.generate(rules,api_level,revision)



def generate(dirs,api_level):
    f = open("script/maintscript/google-android-build-tools-X-installer.dirs.ex")
    i = f.read()
    o = open(dirs, "w")
    o.write(i.replace("$X",api_level))
    o.close()
    print ":... \033[0;34mGENERATED\033[0m google-android-build-tools-"+api_level+"-installer.dirs"



import re

def generate(rules,api_level,revision):
    f = open(rules,"r")
    i = f.read()
    f.seek(0)
    il = f.readlines()
    f.close()
    match = re.search("PLATFORM_"+api_level+"_VERSION = \d+\+r\d+",i)
    if (match):
        if match.group() == "PLATFORM_"+api_level+"_VERSION = "+api_level+"+r"+revision:
            print "\033[0;32mOK\033[0m google-android-platform-"+api_level+"-installer in d/rules"
        else:
            f = open(rules)
            i = f.read()
            o = open(rules, "w")
            i = i.replace(match.group(),"PLATFORM_"+api_level+"_VERSION = "+api_level+"+r"+revision)
            o.write(i)
            o.close()
            print ":... \033[0;34mUPDATED\033[0m google-android-platform-"+api_level+"-installer to revision "+revision
    else:
        print("\033[0;31mNOT EXIST\033[0m google-android-platform-"+api_level+"-installer in d/rules")
        il.insert(1,"\nPLATFORM_"+api_level+"_VERSION = "+api_level+"+r"+revision)
        il.insert(len(il),"\tdh_gencontrol -pgoogle-android-platform-"+api_level+"-installer -- -v$(PLATFORM_"+api_level+"_VERSION) -Tdebian/substvars\n")
        w = open(rules, "w")
        il = "".join(il)
        w.write(il)
        w.close()
        print ":... \033[0;34mGENERATED\033[0m added to d/rules"




def generate(postinst,api_level,archive):
    f = open("script/maintscript/google-android-platform-X-installer.postinst.ex")
    i = f.read()
    o = open(postinst, "w")
    i = i.replace("$X",api_level).replace("$Y",archive)
    o.write(i)
    o.close()
    print ":... \033[0;34mGENERATED\033[0m google-android-platform-"+api_level+"-installer.postinst"



import re, os.path, glob

def get(soup,pif):
    pkg_dir = os.path.join(glob.glob(os.path.expanduser(pif))[0], '')
    #Get m2repository informations
    m2repository_archive = soup.extra.archives.archive

    archive = m2repository_archive.url.string
    sha1 = m2repository_archive.checksum.string

    postinst = pkg_dir+"debian/google-android-m2repository-installer.postinst"
    install = pkg_dir+"debian/google-android-m2repository-installer.install"
    sha1sum = pkg_dir+"for-postinst/default/"+archive+".sha1"
    rules = pkg_dir+"debian/rules"
    current_sha1sum = ""

    revision = soup.extra.revision.major.string
    print "\033[1;34m- Android M2 Repository\033[0m ("+revision+")"

    # Generate/Update <package>.install
    if os.path.isfile(install):
        f = open(install)
        current_sha1sum = re.search("android_m2repository.+.zip.sha1",f.readlines()[1]).group()
        if current_sha1sum == archive+".sha1":
            print("\033[0;32mOK\033[0m google-android-m2repository-installer.install")
        else:
            print("\033[0;33mOUTDATED\033[0m google-android-m2repository-installer.install")
            f.seek(0)
            i = f.read()
            o = open(install,"w")
            o.write(i.replace(current_sha1sum, archive+".sha1"))
            print(":... \033[0;34mUPDATED\033[0m from "+current_sha1sum+" to "+archive)
            o.close()
    else:
         print("\033[0;31mNOT EXIST\033[0m google-android-m2repository-installer.install")

    # Generate/Update <archive>.sha1
    current_sha1sum_file = pkg_dir+"for-postinst/default/"+current_sha1sum
    generate_sha1 = False
    if current_sha1sum != "":
        if os.path.isfile(current_sha1sum_file):
            f = open(current_sha1sum_file)
            current_sha1 = re.search(r'\b[0-9a-f]{5,40}\b',f.readlines()[0]).group()
            if current_sha1sum_file != sha1sum:
                # Remove outdated sha1 file
                try:
		    os.remove(current_sha1sum_file)
	        except OSError:
		    pass
                # Generate new sha1 file
                if os.path.isfile(sha1sum):
                    print("\033[0;32mOK\033[0m "+archive+".sha1")
                else:
                    generate_sha1 = True
            elif current_sha1 != sha1:
                generate_sha1 = True
        else:
            generate_sha1 = True
    else:
        generate_sha1 = True

    #Generate SHA1 if needed
    if generate_sha1 == True:
        i = open(pkg_dir+"for-postinst/default/"+archive+".sha1", "w+")
        i.write(sha1+"  "+archive)
        i.close()
        print ":... \033[0;34mGENERATED\033[0m "+archive+".sha1"

    # Generate/Update <package>.postinst
    if os.path.isfile(postinst):
        f = open(postinst)
        match = re.search("android_m2repository.+.zip",f.readlines()[7]).group()
        if match == archive:
            print("\033[0;32mOK\033[0m google-android-m2repository-installer.postinst")
        else:
            print("\033[0;33mOUTDATED\033[0m google-android-m2repository-installer.postinst")
            f.seek(0)
            i = f.read()
            o = open(postinst,"w")
            o.write(i.replace(match, archive))
            print(":... \033[0;34mUPDATED\033[0m from "+match+" to "+archive)
            o.close()
    else:
        print("\033[0;31mNOT EXIST\033[0m google-android-m2repository-installer.postinst")

    # Update d/rules
    f = open(rules,"r")
    i = f.read()
    f.close()
    match = re.search("M2REPOSITORY_VERSION = \d+",i)
    if match.group() == "M2REPOSITORY_VERSION = "+revision:
        print "\033[0;32mOK\033[0m google-android-m2repository-installer in d/rules"
    else:
        o = open(rules, "w")
        i = i.replace(match.group(),"M2REPOSITORY_VERSION = "+revision)
        o.write(i)
        o.close()
        print ":... \033[0;34mUPDATED\033[0m google-android-m2repository-installer to "+revision



import re, os.path, glob
import maintscript.buildtools_install, maintscript.buildtools_postinst, maintscript.buildtools_postrm, maintscript.buildtools_config, maintscript.buildtools_templates, maintscript.buildtools_dirs, maintscript.buildtools_control, maintscript.buildtools_lintianoverrides, maintscript.buildtools_rules

pkg_dir = ""

def get(soup,pif):
    pkg_dir = os.path.join(glob.glob(os.path.expanduser(pif))[0], '')
    # Get build-tools list
    buildtools_list = soup.findAll('build-tool') 

    # Show results
    latest = buildtools_list[0]
    for bt in buildtools_list:
        if int(bt.revision.major.string) < int (latest.revision.major.string):
            gen(latest)
            latest = bt
            if bt == buildtools_list[-1]:
                gen(latest)
        elif int(bt.revision.minor.string) > int (latest.revision.minor.string):
            latest = bt
        elif int(bt.revision.micro.string) > int (latest.revision.micro.string):
            latest = bt

def gen(buildtools):            
        archive = buildtools.archives.archive.url.string
        sha1 =  buildtools.archives.archive.checksum.string
        api_level = buildtools.revision.major.string
        binary = "google-android-build-tools-"+api_level+"-installer"
        install = pkg_dir+"debian/"+binary+".install"
        postinst = pkg_dir+"debian/"+binary+".postinst"
        postrm = pkg_dir+"debian/"+binary+".postrm"
        config = pkg_dir+"debian/"+binary+".config"
        templates = pkg_dir+"debian/"+binary+".templates"
        dirs = pkg_dir+"debian/"+binary+".dirs"
        overrides = pkg_dir+"debian/"+binary+".lintian-overrides"
        control = pkg_dir+"debian/control"
        rules = pkg_dir+"debian/rules"
        sha1sum = pkg_dir+"for-postinst/default/"+archive+".sha1"
        current_sha1sum = ""
    
        version = api_level+"."+buildtools.revision.minor.string+"."+buildtools.revision.micro.string
        print(("\033[1;32m- Google build tools "+api_level+" for Android \033[0m ("+version+")"))

        # Generate/Update <package>.install
        if os.path.isfile(install):
            f = open(install)
            current_sha1sum = re.search(r"build-tools_r.+-linux.zip.sha1",f.readlines()[1]).group()
            if current_sha1sum == archive+".sha1":
                print("\033[0;32mOK\033[0m "+binary+".install")
            else:
                print("\033[0;33mOUTDATED\033[0m "+binary+".install")
                f.seek(0)
                i = f.read()
                o = open(install,"w")
                o.write(i.replace(current_sha1sum, archive+".sha1"))
                print(":... \033[0;34mUPDATED\033[0m from "+current_sha1sum+" to "+archive+".sha1")
                o.close()
        else:
            print("\033[0;31mNOT EXIST\033[0m "+binary+".install")
            maintscript.buildtools_install.generate(install,api_level,archive)       

        # Generate/Update <archive>.sha1
        current_sha1sum_file = pkg_dir+"for-postinst/default/"+current_sha1sum
        generate_sha1 = False
        if current_sha1sum != "":
           if os.path.isfile(current_sha1sum_file):
               f = open(current_sha1sum_file)
               current_sha1 = re.search(r'\b[0-9a-f]{5,40}\b',f.readlines()[0]).group()
               if current_sha1sum_file != sha1sum:
                   # Remove outdated sha1 file
                   try:
                       os.remove(current_sha1sum_file)
    	           except OSError:
                       pass
                   # Generate new sha1 file
                   if os.path.isfile(sha1sum):
                       print("\033[0;32mOK\033[0m "+archive+".sha1")
                   else:
                       generate_sha1 = True
               elif current_sha1 != sha1:
                   generate_sha1 = True
           else:
               generate_sha1 = True
        else:
           generate_sha1 = True

        # Generate SHA1 if needed
        if generate_sha1 == True:
            i = open(pkg_dir+"for-postinst/default/"+archive+".sha1", "w+")
            i.write(sha1+"  "+archive)
            i.close()
            print ":... \033[0;34mGENERATED\033[0m "+archive+".sha1"
    
        # Generate/Update <package>.postinst
        if os.path.isfile(postinst):
            f = open(postinst)
            match = re.search(r"build-tools_r.+-linux.zip",f.readlines()[7]).group()
            if match == archive:
                print("\033[0;32mOK\033[0m "+binary+".postinst")
            else:
                print("\033[0;33mOUTDATED\033[0m "+binary+".postinst")
                f.seek(0)
                i = f.read()
                o = open(postinst,"w")
                o.write(i.replace(match, archive))
                print(":... \033[0;34mUPDATED\033[0m from revision  to "+archive)
                o.close()
        else:
            print("\033[0;31mNOT EXIST\033[0m "+binary+".postinst")
            maintscript.buildtools_postinst.generate(postinst,api_level,archive,version)
    
        # Generate <package>.postrm
        if os.path.isfile(postrm):
           print("\033[0;32mOK\033[0m "+binary+".postrm")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".postrm")
           maintscript.buildtools_postrm.generate(postrm,api_level)

        #Generate <package>.config
        if os.path.isfile(config):
           print("\033[0;32mOK\033[0m "+binary+".config")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".config")
           maintscript.buildtools_config.generate(config,api_level)

        # Generate <package>.templates
        if os.path.isfile(templates):
           print("\033[0;32mOK\033[0m "+binary+".templates")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".templates")
           maintscript.buildtools_templates.generate(templates,api_level)

        # Generate <package>.dirs
        if os.path.isfile(dirs):
           print("\033[0;32mOK\033[0m "+binary+".dirs")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".dirs")
           maintscript.buildtools_dirs.generate(dirs,api_level)

        #Generate <package>.lintian-overrides
        if os.path.isfile(overrides):
           print("\033[0;32mOK\033[0m "+binary+".lintian-overrides")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+".lintian-overrides")
           maintscript.buildtools_lintianoverrides.generate(overrides,api_level)

        # Add package to d/control
        if "build-tools-"+api_level+"-" in open(control).read():
           print("\033[0;32mOK\033[0m "+binary+" in d/control")
        else:
           print("\033[0;31mNOT EXIST\033[0m "+binary+" in d/control")
           maintscript.buildtools_control.generate(control,api_level,archive)

        # Add package to d/rules
        maintscript.buildtools_rules.generate(rules,api_level,version)



#!/usr/bin/env python
import urllib3, sys, argparse, re, datetime, time, subprocess
import script.platforms, script.ndk, script.docs, script.m2repository, script.buildtools
from bs4 import BeautifulSoup, Comment

#Metadata
progversion = "0.1"
progname="platforms-fetcher"
progdesc="Parser for repository-11.xml to fetch SDK Platforms list"

parser = argparse.ArgumentParser(prog=progname, description=progdesc)
args = parser.parse_args()

url = ""
print "Select a mirror:"
print "[1]: https://dl.google.com"
print "[2]: http://mirrors.neusoft.edu.cn"
print "[3]: http://android-mirror.bugly.qq.com:8080"
while True:
    choice = raw_input("Enter the choice [1/2/3] (default: 1) : ")
    if choice == "1" or choice == "":
        repo_url = "https://dl.google.com/android/repository/repository-12.xml"
        addon_url = "https://dl.google.com/android/repository/addon.xml"
	break
    elif choice == "2":
        repo_url = "http://mirrors.neusoft.edu.cn/android/repository/repository-12.xml"
        addon_url = "http://mirrors.neusoft.edu.cn/android/repository/addon.xml"
	break
    elif choice == "3":
        repo_url = "http://android-mirror.bugly.qq.com:8080/android/repository/repository-12.xml"
        addon_url = "http://android-mirror.bugly.qq.com:8080/android/repository/addon.xml"
        break
    else:
        print "Wrong choice.."

hdr = {"User-Agent": "Mozilla/5.0"}
http = urllib3.PoolManager()

#Fetch Repository-11 XML File
print(("Fetching "+repo_url))
req = http.request('GET',repo_url,headers=hdr)
if req.status != 200:
   print("HTTP Error: %s" % req.status)
   quit()
repo = BeautifulSoup(req.data, "xml")

#Fetch Addon XML File
print(("Fetching "+addon_url))
req = http.request('GET',addon_url,headers=hdr)
if req.status != 200:
   print("HTTP Error: %s" % req.status)
   quit()
addon = BeautifulSoup(req.data, "xml")

#Get Repository-11 Version
gen_comment = str(repo.findAll(text=lambda text:isinstance(text, Comment))[1])
gen_dt = re.findall(r"\d+",gen_comment)
repo_dt = datetime.datetime(int(gen_dt[0]),int(gen_dt[1]),int(gen_dt[2]),int(gen_dt[3]),int(gen_dt[4]),int(gen_dt[5]),int(gen_dt[6]))
repo_live_version = str(int(time.mktime(repo_dt.timetuple())))
print "* Repository Version: \033[2;36m"+repo_live_version+"\033[0m ("+str(repo_dt)+")"

#Get Addon Version
gen_comment = str(addon.findAll(text=lambda text:isinstance(text, Comment))[0])
gen_dt = re.findall(r"\d+",gen_comment)
repo_dt = datetime.datetime(int(gen_dt[0]),int(gen_dt[1]),int(gen_dt[2]),int(gen_dt[3]),int(gen_dt[4]),int(gen_dt[5]),int(gen_dt[6]))
addon_live_version = str(int(time.mktime(repo_dt.timetuple())))
print "* Addon Version: \033[2;96m"+addon_live_version+"\033[0m ("+str(repo_dt)+")"

max_live = 0
if int(repo_live_version) > int(addon_live_version):
    max_live = int(repo_live_version)
else:
    max_live = int(addon_live_version)

o = open('debian/changelog')
current_version = re.search(r"\d+",o.readline()).group()

if int(current_version) == max_live:
	print "* Package Version: \033[2;32m"+current_version+"\033[0m"
else:
	print "* Package Version: \033[0;31m"+current_version+"\033[0m (Version update to \033[1;4m"+str(max_live)+"\033[0m is suggested)"

#Get results
script.platforms.get(repo,".")
script.ndk.get(repo,".")
script.docs.get(repo,".")
script.m2repository.get(addon,".")
script.buildtools.get(repo,".")

#Removing .pyc files
subprocess.call(["pyclean", "."])
